HorschOptipackDD = {};


function HorschOptipackDD.prerequisitesPresent(specializations)
    return true;
end;

function HorschOptipackDD:load(xmlFile)
	self.connectCollisions = SpecializationUtil.callSpecializationsFunction("connectCollisions");
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
	self.Go = {}; 
	self.Done = {};
	self.charId = {};
	self.clipIndex = {};
	self.CheckDone = {};
	self.moveColli = {}; 
    self.animationParts = {};
	self.collisionArm = {};
	self.startpoint = {};
	self.loopCheck = {};
	self.Speed = {};
	local count = getXMLInt(xmlFile, "vehicle.animationParts#count");
	local part = self.animationParts;
	if count ~= nil then
	    for i=1, count do
			part[i] = {};
	        local partname = string.format("vehicle.animationParts.part".."%d", i);
		    local nameR = getXMLString(xmlFile, partname.."#name");
			self.charId[nameR] = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#rootNode"));
			self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
			self.CheckDone[nameR] = false;
			self.moveColli[nameR] = getXMLString(xmlFile, partname.."#moveColli");
			self.startpoint[nameR] = Utils.getNoNil(getXMLFloat(xmlFile, partname.."#startpoint"),0.0);
			self.Speed[nameR] = Utils.getNoNil(getXMLFloat(xmlFile, partname.."#speed"),0.0);
			if self.moveColli[nameR] ~= nil then
				self.Collision = {};
				self.Collision.collision = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collision"));
				self.Collision.collisionAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collsionAttacher"));
				self.Collision.armAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.. "#armAttacher"));
				self.Collision.index = 0;
				self.collisionArm[nameR] = self.Collision;
				self:connectCollisions(nameR);
			end; 
			local charId = getAnimCharacterSet(self.charId[nameR]);
			local clipIndex = getAnimClipIndex(charId, self.clipIndex[nameR]);
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackTime(charId, 0, self.startpoint[nameR]);
			enableAnimTrack(charId, 0);
			disableAnimTrack(charId, 0);
			self.loopCheck[nameR] = false;
			self.Speed[nameR] = 1.0;
		end; 
	end; 	
	self.speedRotatingParts = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.speedRotatingParts.speedRotatingPart(%d)", i);
        local index = getXMLString(xmlFile, baseName.. "#index");
        if index == nil then
            break;
        end;
        local node = Utils.indexToObject(self.components, index);
        if node ~= nil then
            local entry = {};
            entry.node = node;
            entry.rotationSpeedScale = getXMLFloat(xmlFile, baseName.."#rotationSpeedScale");
            if entry.rotationSpeedScale == nil then
                entry.rotationSpeedScale = 1.0/Utils.getNoNil(getXMLFloat(xmlFile, baseName.."#radius"), 1);
            end;
            table.insert(self.speedRotatingParts, entry);
        end;
        i = i+1;
    end;
	self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));
	local cultivatorSound = getXMLString(xmlFile, "vehicle.cultivatorSound#file");
    if cultivatorSound ~= nil and cultivatorSound ~= "" then
        cultivatorSound = Utils.getFilename(cultivatorSound, self.baseDirectory);
        self.cultivatorSound = createSample("cultivatorSound");
        loadSample(self.cultivatorSound, cultivatorSound, false);
        self.cultivatorSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.cultivatorSound#pitchOffset"), 0);
        self.cultivatorSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.cultivatorSound#volume"), 1.0);
        self.cultivatorSoundEnabled = false;
    end;

    self.groundParticleSystems = {};
    local psName = "vehicle.groundParticleSystem";
    Utils.loadParticleSystem(xmlFile, self.groundParticleSystems, psName, self.components, false, nil, self.baseDirectory)
    self.groundParticleSystemActive = false;
end;

function HorschOptipackDD:delete()
	Utils.deleteParticleSystem(self.groundParticleSystems);
    if self.cultivatorSound ~= nil then
        delete(self.cultivatorSound);
    end;
end;

function HorschOptipackDD:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HorschOptipackDD:keyEvent(unicode, sym, modifier, isDown)
end; 

function HorschOptipackDD:update(dt) 
	for k, joint in pairs(self.componentJoints) do
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
	end;
	self:anim("HorschOptipackDDHub", false);
	self:anim("HorschOptipackDDFold", false);
	if self:getIsActive() then
        local hasGroundContact = false;
		if not hasGroundContact then
            if self.groundReferenceNode ~= nil then
                local x,y,z = getWorldTranslation(self.groundReferenceNode);
                local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
                if terrainHeight >= y then
                    hasGroundContact = true;
                end;
            end;
        end;
		if hasGroundContact then
			for k,v in pairs(self.speedRotatingParts) do
                rotate(v.node, v.rotationSpeedScale * self.lastSpeedReal * self.movingDirection * dt, 0, 0);
            end;
            if self.cultivatorSound ~= nil and not self.cultivatorSoundEnabled and self:getIsActiveForSound() then
                if self.lastSpeed*3600 > 3 then
                    playSample(self.cultivatorSound, 0, self.cultivatorSoundVolume, 0);
                    setSamplePitch(self.cultivatorSound, self.cultivatorSoundPitchOffset);
                    self.cultivatorSoundEnabled = true;
                end;
            end;
            if self.lastSpeed*3600 > 5 and not self.groundParticleSystemActive then
                self.groundParticleSystemActive = true;
                Utils.setEmittingState(self.groundParticleSystems, true);
            end;
            if self.lastSpeed*3600 < 5 and self.groundParticleSystemActive then
                self.groundParticleSystemActive = false;
                Utils.setEmittingState(self.groundParticleSystems, false);
            end;
		else
			if self.cultivatorSoundEnabled then
                stopSample(self.cultivatorSound);
                self.cultivatorSoundEnabled = false;
            end;
            if self.groundParticleSystemActive then
                self.groundParticleSystemActive = false;
                Utils.setEmittingState(self.groundParticleSystems, false);
            end;
		end;
	else
		if self.groundParticleSystemActive then
            self.groundParticleSystemActive = false;
            Utils.setEmittingState(self.groundParticleSystems, false);
        end;
	end;
	if self.cultivatorSoundEnabled then
        if self.lastSpeed*3600 < 3 then
            stopSample(self.cultivatorSound);
            self.cultivatorSoundEnabled = false;
        end;
    end;
	if self:getIsActiveForInput() then
		self.Speed.HorschOptipackDDFold = 1;
		self.Speed.HorschOptipackDDHub = 1;
		if self.attacherVehicle ~= nil then
			if self.attacherVehicle.isHorschTiger then
				if self.attacherVehicle.CheckDone.HorschTigerLTFold == true then
					self.Go.HorschOptipackDDFold = true;
					self.Done.HorschOptipackDDFold = true;
					if self.CheckDone.HorschOptipackDDFold == true then
						self.Go.HorschOptipackDDHub = true;
						self.Done.HorschOptipackDDHub = true;
					end;
				else
					if self.CheckDone.HorschOptipackDDHub == false then
						self.Go.HorschOptipackDDFold = false;
						self.Done.HorschOptipackDDFold = true;
					else
						self.Go.HorschOptipackDDHub = false;
						self.Done.HorschOptipackDDHub = true;
					end;
				end;
			else
				if InputBinding.hasEvent(InputBinding.HorschOptipackDD_FOLD) then
					if self.CheckDone.HorschOptipackDDHub == true then
						self.Go.HorschOptipackDDHub = false;
						self.Done.HorschOptipackDDHub = true;
					end;
					self.Go.HorschOptipackDDFold = not self.Go.HorschOptipackDDFold;
					self.Done.HorschOptipackDDFold = true;
				end;
				if self.CheckDone.HorschOptipackDDFold == true then
					if InputBinding.hasEvent(InputBinding.HorschOptipackDD_HUB) then
						self.Go.HorschOptipackDDHub = not self.Go.HorschOptipackDDHub;
						self.Done.HorschOptipackDDHub = true;
					end;
				end;
			end;
		end;
	end;
end;

function HorschOptipackDD:draw()
	if self.attacherVehicle ~= nil then
		if not self.attacherVehicle.isHorschTiger then
			if self.CheckDone.HorschOptipackDDFold == true then
				if self.CheckDone.HorschOptipackDDHub == true then
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschOptipackDD_1"), InputBinding.HorschOptipackDD_HUB);
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("HorschOptipackDD_2"), InputBinding.HorschOptipackDD_HUB);
				end;
			end;
			if self.CheckDone.HorschOptipackDDFold == true then
				g_currentMission:addHelpButtonText(g_i18n:getText("HorschOptipackDD_3"), InputBinding.HorschOptipackDD_FOLD);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("HorschOptipackDD_4"), InputBinding.HorschOptipackDD_FOLD);
			end;
		end;
	end;
end;

function HorschOptipackDD:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	local loopCheck = self.loopCheck[varName];
	local speed = Utils.getNoNil(self.Speed[varName],1);
	local offset = 10;  
	local charId = getAnimCharacterSet(self.charId[varName]);
	local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
	if self.Done[varName] ~= false then
		if self.Go[varName] == true then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			setAnimTrackSpeedScale(charId, 0, speed);
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];	
			end;
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck then
				local setTime = getAnimClipDuration(charId, clipIndex) - getAnimClipDuration(charId, clipIndex);
				setAnimTrackTime(charId, 0, setTime);
			end;
		elseif self.Go[varName] == false then
			if loopCheck == true then
				self.Done[varName] = false;	
			else
				assignAnimTrackClip(charId , 0, clipIndex);
				setAnimTrackLoopState(charId, 0, loopCheck);
				setAnimTrackSpeedScale(charId, 0, -speed);
				enableAnimTrack(charId, 0);
				if getAnimTrackTime(charId, 0) <= 0 then
					self.Done[varName] = false;	
					self.CheckDone[varName] = self.Go[varName];
				end;
			end;
		elseif self.Go[varName] ~= nil then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			if self.Go[varName] < 0.0 then
				self.Go[varName] = 0.0;
			elseif self.Go[varName] > getAnimClipDuration(charId, clipIndex) then
				self.Go[varName] = getAnimClipDuration(charId, clipIndex);
			end;
			if getAnimTrackTime(charId, 0) >= self.Go[varName] then
				setAnimTrackSpeedScale(charId, 0, -speed);
			else
				setAnimTrackSpeedScale(charId, 0, speed);
			end;
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) <= self.Go[varName] + offset and getAnimTrackTime(charId, 0) >= self.Go[varName] - offset then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];
			end;
		end;
	end;
	if self.Done[varName] == false then
		disableAnimTrack(charId, 0);
	end;
end; 

function HorschOptipackDD:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;

function HorschOptipackDD:onDetach()
	if self.groundParticleSystemActive then
		self.groundParticleSystemActive = false;
		Utils.setEmittingState(self.groundParticleSystems, false);
	end;
	if self.cultivatorSoundEnabled then
        stopSample(self.cultivatorSound);
        self.cultivatorSoundEnabled = false;
    end;
end;

function HorschOptipackDD:getSaveAttributesAndNodes(nodeIdent)
	if self.CheckDone.HorschOptipackDDFold then
		trsp = "true";
	else
		trsp = "false";
	end; 	
	local attributes = 'transport="'..trsp..'"';
	return attributes, nil;
end;

function HorschOptipackDD:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    local trsp = getXMLString(xmlFile, key.."#transport");
	if trsp == "true" and not resetVehicles then
		self.Speed.HorschOptipackDDFold = 100;
		self.Go.HorschOptipackDDFold = true;
		self.Done.HorschOptipackDDFold = true;
		self.Speed.HorschOptipackDDHub = 20;
		self.Go.HorschOptipackDDHub = true;
		self.Done.HorschOptipackDDHub = true;
	end; 
    return BaseMission.VEHICLE_LOAD_OK;
end;